package de.fhws.robotxkey.txc;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.regex.Pattern;

public class TxcProtocol {
	public boolean boolValid = false;
	
	public int intMagic = 0;
	public int intLength = 0;
	public int[] intData = null;
	public int intChecksum = 0;
	
	public TxcProtocol() {
		{
			this.boolValid = false;
		}
		
		{
			this.intMagic = 0;
			this.intLength = 0;
			this.intData = null;
			this.intChecksum = 0;
		}
	}
	
	public void finalize() {
		{
			this.boolValid = false;
		}
		
		{
			this.intMagic = 0;
			this.intLength = 0;
			this.intData = null;
			this.intChecksum = 0;
		}
	}
	
	public void availability(InputStream inputstreamHandle, int intTimeout) throws Exception {
		long longTimeout = System.currentTimeMillis() + 3000;
		
		do {
			if (inputstreamHandle.available() != 0) {
				break;
			}
			
			if (System.currentTimeMillis() > longTimeout) {
				throw new Exception();
			}
		} while (true);
	}
	
	public void getConnack(InputStream inputstreamHandle) {
		int[] intConnack = { 'C', 'O', 'N', 'N', '_', 'A', 'C', 'K' };
		
		try {
			{
				this.boolValid = false;
			}
			
			{
				this.read(inputstreamHandle, 3000);
			}
			
			if (this.boolValid == false) {
				throw new Exception();
			}
			
			if (Arrays.equals(this.intData, intConnack) == false) {
				throw new Exception();
			}
			
			{
				this.boolValid = true;
			}
		} catch (Exception e) {
			// e.printStackTrace();
		}
	}
	
	public void getEchocheck(InputStream inputstreamHandle, OutputStream outputstreamHandle) {
		int[] intEchocheck = { 'E', 'C', 'H', 'O', 'C', 'H', 'E', 'C', 'K' };
		
		try {
			{
				this.boolValid = false;
			}
			
			{
				this.intData = Arrays.copyOf(intEchocheck, intEchocheck.length);
				
				this.write(outputstreamHandle);
			}
			
			if (this.boolValid == false) {
				throw new Exception();
			}
			
			{
				this.read(inputstreamHandle, 1000);
			}
			
			if (this.boolValid == false) {
				throw new Exception();
			}
			
			if (Arrays.equals(this.intData, intEchocheck) == false) {
				throw new Exception();
			}
			
			{
				this.boolValid = true;
			}
		} catch (Exception e) {
			// e.printStackTrace();
		}
	}
	
	public void read(InputStream inputstreamHandle, int intTimeout) {
		try {
			{
				this.boolValid = false;
			}
			
			{
				int intChecksum = 0;
				
				{
					// Magic
					
					byte[] byteProtocol = new byte[2];
					int[] intProtocol = new int[byteProtocol.length];
					
					this.availability(inputstreamHandle, intTimeout);
					int intRead = inputstreamHandle.read(byteProtocol);
					
					if (intRead != byteProtocol.length) {
						throw new Exception();
					}
					
					for (int intFor1 = 0; intFor1 < byteProtocol.length; intFor1 += 1) {
						if (byteProtocol[intFor1] < 0) {
							intProtocol[intFor1] = byteProtocol[intFor1] + 256;
							
						} else if (byteProtocol[intFor1] < 128) {
							intProtocol[intFor1] = byteProtocol[intFor1] + 0;
							
						}
					}
					
					for (int intFor1 = 0; intFor1 < intProtocol.length; intFor1 += 1) {
						intChecksum += intProtocol[intFor1];
						intChecksum %= 256;
					}
					
					this.intMagic = (intProtocol[0] << 8) + (intProtocol[1] << 0);
				}
				
				if (this.intMagic != 48879) {
					throw new Exception();
				}
				
				{
					// Length
					
					byte[] byteProtocol = new byte[1];
					int[] intProtocol = new int[byteProtocol.length];
					
					this.availability(inputstreamHandle, intTimeout);
					int intRead = inputstreamHandle.read(byteProtocol);
					
					if (intRead != byteProtocol.length) {
						throw new Exception();
					}
					
					for (int intFor1 = 0; intFor1 < byteProtocol.length; intFor1 += 1) {
						if (byteProtocol[intFor1] < 0) {
							intProtocol[intFor1] = byteProtocol[intFor1] + 256;
							
						} else if (byteProtocol[intFor1] < 128) {
							intProtocol[intFor1] = byteProtocol[intFor1] + 0;
							
						}
					}
					
					for (int intFor1 = 0; intFor1 < intProtocol.length; intFor1 += 1) {
						intChecksum += intProtocol[intFor1];
						intChecksum %= 256;
					}
					
					this.intLength = (intProtocol[0] << 0);
				}
				
				{
					// Data
					
					byte[] byteProtocol = new byte[this.intLength];
					int[] intProtocol = new int[byteProtocol.length];
					
					int intRead = 0;
					
					do {
						if (byteProtocol.length == intRead) {
							break;
						}
						
						this.availability(inputstreamHandle, intTimeout);
						int intPart = inputstreamHandle.read(byteProtocol, intRead, byteProtocol.length - intRead);
						
						if (intPart == 0) {
							break;
							
						} else if (intPart == -1) {
							break;
							
						}
						
						intRead += intPart;
					} while (true);
					
					if (intRead != byteProtocol.length) {
						throw new Exception();
					}
					
					for (int intFor1 = 0; intFor1 < byteProtocol.length; intFor1 += 1) {
						if (byteProtocol[intFor1] < 0) {
							intProtocol[intFor1] = byteProtocol[intFor1] + 256;
							
						} else if (byteProtocol[intFor1] < 128) {
							intProtocol[intFor1] = byteProtocol[intFor1] + 0;
							
						}
					}
					
					for (int intFor1 = 0; intFor1 < intProtocol.length; intFor1 += 1) {
						intChecksum += intProtocol[intFor1];
						intChecksum %= 256;
					}
					
					this.intData = Arrays.copyOf(intProtocol, intProtocol.length);
				}
				
				{
					// Checksum
					
					byte[] byteProtocol = new byte[1];
					int[] intProtocol = new int[byteProtocol.length];
					
					this.availability(inputstreamHandle, intTimeout);
					int intRead = inputstreamHandle.read(byteProtocol);
					
					if (intRead != byteProtocol.length) {
						throw new Exception();
					}
					
					for (int intFor1 = 0; intFor1 < byteProtocol.length; intFor1 += 1) {
						if (byteProtocol[intFor1] < 0) {
							intProtocol[intFor1] = byteProtocol[intFor1] + 256;
							
						} else if (byteProtocol[intFor1] < 128) {
							intProtocol[intFor1] = byteProtocol[intFor1] + 0;
							
						}
					}
					
					this.intChecksum = (intProtocol[0] << 0);
				}
				
				if (this.intChecksum != intChecksum) {
					throw new Exception();
				}
			}
			
			{
				this.boolValid = true;
			}
		} catch (Exception e) {
			// e.printStackTrace();
		}
	}
	
	public void write(OutputStream outputstreamHandle) {
		try {
			{
				this.boolValid = false;
			}
			
			{
				int intPosition = 0;
				
				int[] intProtocol = new int[2 + 1 + this.intData.length + 1];
				byte[] byteProtocol = new byte[intProtocol.length];
				
				{
					// Magic
					
					intProtocol[intPosition] = 0xBE;
					intPosition += 1;
					
					intProtocol[intPosition] = 0xEF;
					intPosition += 1;
				}
				
				{
					// Length
					
					intProtocol[intPosition] = this.intData.length;
					intPosition += 1;
				}
				
				{
					// Data
					
					for (int intFor1 = 0; intFor1 < this.intData.length; intFor1 += 1) {
						intProtocol[intPosition] = this.intData[intFor1];
						intPosition += 1;
					}
				}
				
				{
					// Checksum
					
					int intChecksum = 0;
					
					for (int intFor1 = 0; intFor1 < intProtocol.length; intFor1 += 1) {
						intChecksum += intProtocol[intFor1];
						intChecksum %= 256;
					}
					
					intProtocol[intPosition] = intChecksum;
					intPosition += 1;
					
					if (intProtocol[intPosition - 1] < 0) {
						intProtocol[intPosition - 1] = intProtocol[intPosition - 1] + 256;
						
					} else if (intProtocol[intPosition - 1] < 128) {
						intProtocol[intPosition - 1] = intProtocol[intPosition - 1] + 0;
						
					}
				}
				
				for (int intFor1 = 0; intFor1 < intProtocol.length; intFor1 += 1) {
					if (intProtocol[intFor1] < 128) {
						byteProtocol[intFor1] = (byte) (intProtocol[intFor1] - 0);
						
					} else if (intProtocol[intFor1] < 256) {
						byteProtocol[intFor1] = (byte) (intProtocol[intFor1] - 256);
						
					}
				}
				
				outputstreamHandle.write(byteProtocol);
			}
			
			{
				this.boolValid = true;
			}
		} catch (IOException e) {
			// e.printStackTrace();
		}
	}
	
	public void getCorrect(InputStream inputstreamHandle, OutputStream outputstreamHandle) {
		try {
			{
				this.boolValid = false;
			}
			
			String strInput = "";
			String strOutput = "";
			
			BufferedReader bufferedreaderHandle = new BufferedReader(new InputStreamReader(inputstreamHandle));
			
			{
				// stop the program on the controller - what sadly isn't possible ...
			}
			
			{
				{
					strOutput = "";
					
					strOutput += "set_ebd this";
					strOutput += "\r\n";
					
					outputstreamHandle.write(strOutput.getBytes());
				}
				
				{
					Thread.sleep(5000);
				}
				
				{
					char[] charRead = new char[1024];
					int intRead = bufferedreaderHandle.read(charRead);
					
					strInput += String.valueOf(charRead).substring(0, intRead);
				}
				
				{
					String[] strSplit = strInput.split(Pattern.quote(strOutput));
					
					if (strSplit.length != 2) {
						throw new Exception();
					}
					
					strInput = strSplit[1];
				}
				
				{
					String[] strSplit = strInput.split(Pattern.quote("SUCCESS"));
					
					if (strSplit.length != 2) {
						throw new Exception();
					}
					
					strInput = strSplit[1];
				}
				
				{
					String[] strSplit = strInput.split(Pattern.quote("ROBO TX"));
					
					if (strSplit.length != 2) {
						throw new Exception();
					}
					
					strInput = strSplit[1];
				}
				
				{
					String[] strSplit = strInput.split(Pattern.quote(">"));
					
					if (strSplit.length != 2) {
						throw new Exception();
					}
					
					strInput = strSplit[1];
				}
			}
			
			{
				// start the program on the controller - what sadly isn't possible ...
			}
			
			{
				this.boolValid = true;
			}
		} catch (Exception e) {
			// e.printStackTrace();
		}
	}
}
